#include <Wire.h>
#include <TM1650.h>
#include <HX711_ADC.h>

#if defined(ESP8266)|| defined(ESP32) || defined(AVR)
#include <EEPROM.h>
#endif


//HX711 pins:
const int HX711_dout = 04; //mcu > HX711 dout pin
const int HX711_sck = 05; //mcu > HX711 sck pin


TM1650 d;

HX711_ADC LoadCell(HX711_dout, HX711_sck);

float newCalibrationValue;
float pesoInhg;
float g = 0;
String pesoInhgs;
char line[5] = "  00";

const long interval = 20000;  //Durata ciclo accensione della bilancia

const int calVal_eepromAdress = 0;
unsigned long t = 0;

void setup() {

  pinMode(10 , OUTPUT); // mantiene acceso il circuito dopo il rilascio del touch sensor
  digitalWrite(10, HIGH);
  
  Wire.begin(); //Start the I2C bus as master

  Serial.begin(57600); 
  Serial.println("Start");

  
  d.init();
  d.displayOff();
  d.displayString("____");
  d.setBrightness(3);
  d.displayOn();

  
  LoadCell.begin();
  LoadCell.setReverseOutput(); //uncomment to turn a negative output value to positive
  unsigned long stabilizingtime = 2000; // precision right after power-up can be improved by adding a few seconds of stabilizing time
  boolean _tare = true; //set this to false if you don't want tare to be performed in the next step
  LoadCell.start(stabilizingtime, _tare);
  if (LoadCell.getTareTimeoutFlag() || LoadCell.getSignalTimeoutFlag()) {
    Serial.println(F("Timeout, check MCU>HX711 wiring and pin designations"));
    while (1);
 
  }
  else {
    EEPROM.get(calVal_eepromAdress, newCalibrationValue);
    LoadCell.setCalFactor(newCalibrationValue); // user set calibration value (float)
    Serial.println(F("Startup is complete"));
  }
  while (!LoadCell.update());

}

void loop() 
{
    static boolean newDataReady = false;
    const int serialPrintInterval = 100; //intervallo (in ms) tra letture successive

    // check for new data/start next conversion:
    if (LoadCell.update()) newDataReady = true;

    // get smoothed value from the dataset:
    if (newDataReady) {
      if (millis() > t + serialPrintInterval) {
        float g = LoadCell.getData(); // peso in grammi


    pesoInhg = g/100;
    //Serial.println(pesoInhg);
    if (g < 1000) {
      pesoInhgs = ("  " + String(pesoInhg,0));
    }
    
    else if (g < 9950) {
      pesoInhgs = ("  " + String(pesoInhg,0));
    }

    else if (g < 99950) {
      pesoInhgs = (" " + String(pesoInhg,0));
    }
    else {
      pesoInhgs = (String(pesoInhg,0));
    }
    
    
    pesoInhgs.toCharArray(line,5);
    if (pesoInhg < 9.5){
      line[2] = '0';
    }
    //Serial.println(line); // solo per debug
    
    d.displayString(line);
    d.setDot(2,true);
    delay(30); // necessario per evitare sfarfallio punto decimale
      }

        // receive command from serial terminal
    if (Serial.available() > 0) {
      char inByte = Serial.read();
      if (inByte == 't') LoadCell.tareNoDelay(); //tare
      else if (inByte == 'r') calibrate(); //calibrate
      else if (inByte == 'c') changeSavedCalFactor(); //edit calibration value manually
    }

    // check if last tare operation is complete
    if (LoadCell.getTareStatus() == true) {
      Serial.println(F("Tare complete"));
      }
    } 
    unsigned long currentMillis = millis();

    if (currentMillis >= interval){  
      digitalWrite(10, LOW); // spegne il circuito allo scadere del tempo stabilito
    }
    
}


void calibrate() {
  Serial.println(F("***"));
  Serial.println(F("Start calibration:"));
  Serial.println(F("Place the load cell an a level stable surface."));
  Serial.println(F("Remove any load applied to the load cell."));
  Serial.println(F("Send 't' from serial monitor to set the tare offset."));

  boolean _resume = false;
  while (_resume == false) {
    LoadCell.update();
    if (Serial.available() > 0) {
      if (Serial.available() > 0) {
        char inByte = Serial.read();
        if (inByte == 't') LoadCell.tareNoDelay();
      }
    }
    if (LoadCell.getTareStatus() == true) {
      Serial.println(F("Tare complete"));
      _resume = true;
    }
  }

  Serial.println(F("Now, place your known mass on the loadcell."));
  Serial.println(F("Then send the weight of this mass (in grams) (i.e. 100.0) from serial monitor."));

  float known_mass = 0;
  _resume = false;
  while (_resume == false) {
    LoadCell.update();
    if (Serial.available() > 0) {
      known_mass = Serial.parseFloat();
      if (known_mass != 0) {
        Serial.print(F("Known mass is: "));
        Serial.println(known_mass);
        _resume = true;
      }
    }
  }

  LoadCell.refreshDataSet(); //refresh the dataset to be sure that the known mass is measured correct
  float newCalibrationValue = LoadCell.getNewCalibration(known_mass); //get the new calibration value

  Serial.print(F("New calibration value has been set to: "));
  Serial.print(newCalibrationValue);
  Serial.println(F(", use this as calibration value (calFactor) in your project sketch."));
  Serial.print(F("Save this value to EEPROM adress "));
  Serial.print(calVal_eepromAdress);
  Serial.println(F("? y/n"));

  _resume = false;
  while (_resume == false) {
    if (Serial.available() > 0) {
      char inByte = Serial.read();
      if (inByte == 'y') {
#if defined(ESP8266)|| defined(ESP32)
        EEPROM.begin(512);
#endif
        EEPROM.put(calVal_eepromAdress, newCalibrationValue);
#if defined(ESP8266)|| defined(ESP32)
        EEPROM.commit();
#endif
        EEPROM.get(calVal_eepromAdress, newCalibrationValue);
        Serial.print(F("Value "));
        Serial.print(newCalibrationValue);
        Serial.print(F(" saved to EEPROM address: "));
        Serial.println(calVal_eepromAdress);
        _resume = true;

      }
      else if (inByte == 'n') {
        Serial.println(F("Value not saved to EEPROM"));
        _resume = true;
      }
    }
  }

  Serial.println(F("End calibration"));
  Serial.println(F("***"));
  Serial.println(F("To re-calibrate, send 'r' from serial monitor."));
  Serial.println(F("For manual edit of the calibration value, send 'c' from serial monitor."));
  Serial.println(F("***"));
}

void changeSavedCalFactor() {
  float oldCalibrationValue = LoadCell.getCalFactor();
  boolean _resume = false;
  Serial.println(F("***"));
  Serial.print(F("Current value is: "));
  Serial.println(oldCalibrationValue);
  Serial.println(F("Now, send the new value from serial monitor, i.e. 696.0"));
  //float newCalibrationValue; // definizione spostata all'inizio per utilizzare il valore memorizzato in EEPROM nel setup
  while (_resume == false) {  
    if (Serial.available() > 0) {
      newCalibrationValue = Serial.parseFloat();
      if (newCalibrationValue != 0) {
        Serial.print(F("New calibration value is: "));
        Serial.println(newCalibrationValue);
        LoadCell.setCalFactor(newCalibrationValue);
        _resume = true;
      }
    }
  }
  _resume = false;
  Serial.print(F("Save this value to EEPROM adress "));
  Serial.print(calVal_eepromAdress);
  Serial.println(F("? y/n"));
  while (_resume == false) {
    if (Serial.available() > 0) {
      char inByte = Serial.read();
      if (inByte == 'y') {
#if defined(ESP8266)|| defined(ESP32)
        EEPROM.begin(512);
#endif
        EEPROM.put(calVal_eepromAdress, newCalibrationValue);
#if defined(ESP8266)|| defined(ESP32)
        EEPROM.commit();
#endif
        EEPROM.get(calVal_eepromAdress, newCalibrationValue);
        Serial.print(F("Value "));
        Serial.print(newCalibrationValue);
        Serial.print(F(" saved to EEPROM address: "));
        Serial.println(calVal_eepromAdress);
        _resume = true;
      }
      else if (inByte == 'n') {
        Serial.println(F("Value not saved to EEPROM"));
        _resume = true;
      }
    }
  }
  Serial.println(F("End change calibration value"));
  Serial.println(F("***"));
}
